@@ -1,3 +1,11 @@
+/* This file is a simple Unicode library dealing with conversions to
+   and from the UTF-8 format. */
+
+/* 
+   Author: Ben Bullock <benkasminbullock@gmail.com>. 
+   Repository: https://github.com/benkasminbullock/unicode-c
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -8,27 +16,58 @@
 /* This length includes one trailing nul byte. */
 #define UTF8_MAX_LENGTH 5
 #define UNICODE_BAD_INPUT -1
+
+/* An illegal surrogate pair code was attempted to turn into UTF-8. */
+
 #define UNICODE_SURROGATE_PAIR -2
+
+/* Values not forming a surrogate pair were tried to be converted as
+   if they were a surrogate pair. */
+
 #define UNICODE_NOT_SURROGATE_PAIR -3
+
+/* Input which was supposed to be UTF-8 encoded was not. */
+
 #define UNICODE_BAD_UTF8 -4
+
+/* A string of UTF-8 bytes turned out to contain a zero as its first
+   byte. */
+
 #define UNICODE_EMPTY_INPUT -5
+
+/* Some UTF-8 bytes were not in the shortest possible form. */
+
 #define UNICODE_NON_SHORTEST -6
+
+/* There was an attempt to convert a code point which was greater than
+   UNICODE_UTF8_4 into UTF-8 bytes. */
+
 #define UNICODE_TOO_BIG -7
+
+/* The Unicode code-point ended with either 0xFFFF or 0xFFFE, meaning
+   it cannot e used as a character code point. */
+
 #define UNICODE_NOT_CHARACTER -8
 
-/* The maximum value which is allowed. */
+/* The maximum possible value of a Unicode code point. */
 
 #define UNICODE_MAXIMUM 0x10ffff
 
-/* The maximum value which will fit into four bytes of UTF-8. */
+/* The maximum possible value which will fit into four bytes of
+   UTF-8. This is larger than UNICODE_MAXIMUM. */
 
 #define UNICODE_UTF8_4 0x1fffff
 
-/* For routines which don't need a return value. */
+/* For routines which don't use the return value to communicate back
+   to the caller, this return value indicates a successful
+   completion. */
+
 #define UNICODE_OK 0
 #endif /* def HEADER */
 
-/* How many bytes to expect. */
+/* The return value is how many bytes of UTF-8 character point "c"
+   will become. If "c" is not a valid UTF-8 byte, the value
+   UNICODE_BAD_UTF8 is returned. */
 
 int utf8_bytes (unsigned char c)
 {
@@ -47,9 +86,11 @@ int utf8_bytes (unsigned char c)
     return UNICODE_BAD_UTF8;
 }
 
-/* Give the values even if the input is partly broken. */
+/* Try to convert "input" from UTF-8 to UCS-2, and return a value even
+   if the input is partly broken. */
 
-int utf8_no_checks (const unsigned char * input, const unsigned char ** end_ptr)
+int
+utf8_no_checks (const unsigned char * input, const unsigned char ** end_ptr)
 {
     unsigned char c;
     c = input[0];
@@ -89,9 +130,10 @@ int utf8_no_checks (const unsigned char * input, const unsigned char ** end_ptr)
 
 /* Convert a UTF-8 encoded character in "input" into a number. This
    function returns the unicode value of the UTF-8 character if
-   successful, and -1 if not successful. "end_ptr" is set to the next
-   character after the read character on success. "end_ptr" is set to
-   the start of input on failure. "end_ptr" may not be null. */
+   successful, and a negative number if not successful. "end_ptr" is
+   set to the next character after the read character on
+   success. "end_ptr" is set to the start of input on
+   failure. "end_ptr" may not be null. */
 
 int utf8_to_ucs2 (const unsigned char * input, const unsigned char ** end_ptr)
 {
@@ -186,7 +228,8 @@ int utf8_to_ucs2 (const unsigned char * input, const unsigned char ** end_ptr)
    negative number if there was an error.
 
    This adds a zero byte to the end of the string. It assumes that the
-   buffer "utf8" has at least four bytes of space to write to. */
+   buffer "utf8" has at least UNICODE_MAX_LENGTH (5) bytes of space to
+   write to, without checking. */
 
 int ucs2_to_utf8 (int ucs2, unsigned char * utf8)
 {
@@ -221,7 +264,7 @@ int ucs2_to_utf8 (int ucs2, unsigned char * utf8)
         utf8[4] = '\0';
         return 4;
     }
-    return UNICODE_BAD_INPUT;
+    return UNICODE_TOO_BIG;
 }
 
 /* Convert surrogate pairs to UTF-8. */
@@ -251,6 +294,16 @@ int surrogate_to_utf8 (int hi, int lo, unsigned char * utf8)
 #define UNI_SUR_LOW_START   0xDC00
 #define UNI_SUR_LOW_END     0xDFFF
 
+/* Convert a unicode code point "unicode" into its high and low
+   surrogate pair equivalents, in "*hi_ptr" and "*lo_ptr".
+
+   Return value:
+
+   If "unicode" does not need to be a surrogate pair, the error
+   UNICODE_NOT_SURROGATE_PAIR is returned, and the values of "*hi_ptr"
+   and "*lo_ptr" are undefined. If the conversion is successful,
+   UNICODE_OK is returned. */
+
 int
 unicode_to_surrogates (unsigned unicode, unsigned * hi_ptr, unsigned * lo_ptr)
 {
@@ -269,7 +322,11 @@ unicode_to_surrogates (unsigned unicode, unsigned * hi_ptr, unsigned * lo_ptr)
     return UNICODE_OK;
 }
 
-/* Convert hi/lo surrogate pair to a single Unicode value. */
+/* Convert a surrogate pair in "hi" and "lo" to a single Unicode
+   value. The return value is the Unicode value. If the return value
+   is negative, an error has occurred. If "hi" and "lo" do not form a
+   surrogate pair, the error value UNICODE_NOT_SURROGATE_PAIR (-3) is
+   returned. */
 
 /* https://android.googlesource.com/platform/external/id3lib/+/master/unicode.org/ConvertUTF.c */
 
@@ -291,10 +348,15 @@ surrogates_to_unicode (unsigned hi, unsigned lo)
     return UNICODE_NOT_SURROGATE_PAIR;
 }
 
-/* Given a count of Unicode characters "n_chars", return the number of
-   bytes. A negative value indicates some kind of error. */
+/* Given a nul-terminated string "utf8" and a number of Unicode
+   characters "n_chars", return the number of bytes into "utf8" at
+   which the end of the characters occurs. A negative value indicates
+   some kind of error. If the return value is UNICODE_EMPTY_INPUT,
+   "utf8" contained a zero byte. This may also return any of the
+   values of "utf8_to_ucs2". */
 
-int unicode_chars_to_bytes (const unsigned char * utf8, int n_chars)
+int
+unicode_chars_to_bytes (const unsigned char * utf8, int n_chars)
 {
     int i;
     const unsigned char * p = utf8;
@@ -311,6 +373,14 @@ int unicode_chars_to_bytes (const unsigned char * utf8, int n_chars)
     return p - utf8;
 }
 
+/* Given a nul-terminated string "utf8", return the total number of
+   Unicode characters it contains.
+
+   Return value
+
+   If an error occurs, this may return UNICODE_BAD_INPUT or any of the
+   errors of "utf8_to_ucs2". */
+
 int unicode_count_chars (const unsigned char * utf8)
 {
     int chars = 0;
@@ -322,14 +392,16 @@ int unicode_count_chars (const unsigned char * utf8)
     while (p - utf8 < len) {
         int ucs2;
         ucs2 = utf8_to_ucs2 (p, & p);
-        if (ucs2 == -1) {
-            return -1;
+        if (ucs2 < 0) {
+            return ucs2;
         }
         chars++;
         if (*p == '\0') {
             return chars;
         }
     }
+    /* Cannot be reached in practice, since strlen indicates the null
+       byte. */
     return UNICODE_BAD_INPUT;
 }
 
@@ -395,6 +467,10 @@ int unicode_count_chars (const unsigned char * utf8)
 
 #define UNICODENEXTBYTE c=input[i]
 
+/* Given "input" and "input_length", validate "input" byte by byte up
+   to "input_length". The return value may be VALID_UTF8 or
+   INVALID_UTF8. */
+
 int
 valid_utf8 (const unsigned char * input, int input_length)
 {
@@ -531,6 +607,8 @@ valid_utf8 (const unsigned char * input, int input_length)
     }
 }
 
+/* Below this is code for testing which is not normally compiled. Use
+   "make test" to compile the testing version. */
 
 #ifdef TEST
 
